Package: spatstat.model
Version: 3.7-1
Date: 2026-05-29
Title: Parametric Statistical Modelling and Inference for the
        'spatstat' Family
Authors@R: c(person("Adrian", "Baddeley", 
                    role = c("aut", "cre", "cph"),
       	            email = "Adrian.Baddeley@curtin.edu.au",
		    comment = c(ORCID="0000-0001-9499-8382")),
	     person("Rolf", "Turner", 
                    role = c("aut", "cph"),
 	            email="rolfturner@posteo.net",
		    comment=c(ORCID="0000-0001-5521-5218")),
	     person("Ege",   "Rubak", 
                    role = c("aut", "cph"),
		    email = "rubak@math.aau.dk",
		    comment=c(ORCID="0000-0002-6675-533X")),
	     person("Kasper", "Klitgaard Berthelsen",
	            role = "ctb"),
	     person("Achmad", "Choiruddin",
	            role = c("ctb", "cph")),
	     person("Jean-Francois", "Coeurjolly",
	            role = "ctb"),
	     person("Ottmar", "Cronie",
	            role = "ctb"),
	     person("Tilman", "Davies",
	            role = "ctb"),
	     person("Julian", "Gilbey",
	            role = "ctb"),
	     person("Yongtao", "Guan",
	            role = "ctb"),
	     person("Ute", "Hahn",
	            role = "ctb"),
	     person("Martin", "Hazelton",
	            role = "ctb"),
	     person("Kassel", "Hingee",
	            role = "ctb"),
	     person("Abdollah", "Jalilian", 
	            role = "ctb"),
	     person("Frederic", "Lavancier",
	            role = "ctb"),
	     person("Marie-Colette", "van Lieshout",
	            role = "ctb"),
             person("Bethany", "Macdonald",
	     	    role = "ctb"),
	     person("Greg", "McSwiggan",
	            role = "ctb"),
	     person("Tuomas", "Rajala",
	            role = "ctb"),
	     person("Suman", "Rakshit",
	            role = c("ctb", "cph")),
	     person("Dominic", "Schuhmacher",
	            role = "ctb"),
	     person("Rasmus", "Plenge Waagepetersen",
	            role = "ctb"),
	     person("Hangsheng", "Wang",
	            role = "ctb"))
Maintainer: Adrian Baddeley <Adrian.Baddeley@curtin.edu.au>
Depends: R (>= 3.5.0), spatstat.data (>= 3.1-9), spatstat.univar (>=
        3.2-0), spatstat.geom (>= 3.8-1), spatstat.random (>= 3.5-0),
        spatstat.explore (>= 3.8-1), stats, graphics, grDevices, utils,
        methods, nlme, rpart
Imports: spatstat.utils (>= 3.2-3), spatstat.sparse (>= 3.2-0), mgcv,
        Matrix, abind, tensor, goftest (>= 1.2-2)
Suggests: sm, gsl, locfit, spatial, fftwtools (>= 0.9-8), nleqslv,
        glmnet, spatstat.linnet (>= 3.5-0), spatstat (>= 3.6-0)
Description: Functionality for parametric statistical modelling and inference for spatial data,
	     mainly spatial point patterns, in the 'spatstat' family of packages.
	     (Excludes analysis of spatial data on a linear network,
	     which is covered by the separate package 'spatstat.linnet'.)
	     Supports parametric modelling, formal statistical inference, and model validation.
	     Parametric models include Poisson point processes, Cox point processes, Neyman-Scott cluster processes, Gibbs point processes and determinantal point processes. Models can be fitted to data using maximum likelihood, maximum pseudolikelihood, maximum composite likelihood and the method of minimum contrast. Fitted models can be simulated and predicted. Formal inference includes hypothesis tests (quadrat counting tests, Cressie-Read tests, Clark-Evans test, Berman test, Diggle-Cressie-Loosmore-Ford test, scan test, studentised permutation test, segregation test, ANOVA tests of fitted models, adjusted composite likelihood ratio test, envelope tests, Dao-Genton test, balanced independent two-stage test), confidence intervals for parameters, and prediction intervals for point counts. Model validation techniques include leverage, influence, partial residuals, added variable plots, diagnostic plots, pseudoscore residual plots, model compensators and Q-Q plots.
License: GPL (>= 2)
URL: http://spatstat.org/
NeedsCompilation: yes
ByteCompile: true
BugReports: https://github.com/spatstat/spatstat.model/issues
Packaged: 2026-05-29 02:37:45 UTC; adrian
Author: Adrian Baddeley [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-9499-8382>),
  Rolf Turner [aut, cph] (ORCID: <https://orcid.org/0000-0001-5521-5218>),
  Ege Rubak [aut, cph] (ORCID: <https://orcid.org/0000-0002-6675-533X>),
  Kasper Klitgaard Berthelsen [ctb],
  Achmad Choiruddin [ctb, cph],
  Jean-Francois Coeurjolly [ctb],
  Ottmar Cronie [ctb],
  Tilman Davies [ctb],
  Julian Gilbey [ctb],
  Yongtao Guan [ctb],
  Ute Hahn [ctb],
  Martin Hazelton [ctb],
  Kassel Hingee [ctb],
  Abdollah Jalilian [ctb],
  Frederic Lavancier [ctb],
  Marie-Colette van Lieshout [ctb],
  Bethany Macdonald [ctb],
  Greg McSwiggan [ctb],
  Tuomas Rajala [ctb],
  Suman Rakshit [ctb, cph],
  Dominic Schuhmacher [ctb],
  Rasmus Plenge Waagepetersen [ctb],
  Hangsheng Wang [ctb]
Repository: CRAN
Date/Publication: 2026-05-29 07:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-29 08:03:33 UTC; unix
Archs: spatstat.model.so.dSYM
