## spatstat.model NAMESPACE file

## ................  Import packages ..................

import(stats,graphics,grDevices,utils,methods)
import(spatstat.utils,spatstat.data,
       spatstat.univar,spatstat.sparse,
       spatstat.geom,spatstat.random,spatstat.explore)
import(goftest)
import(Matrix,nlme,rpart)

importFrom(abind,abind)
importFrom(tensor,tensor)
importFrom(mgcv,
           gam,gam.control,anova.gam,formula.gam,predict.gam,
           print.gam,summary.gam,vcov.gam,s)

## ................ Load dynamic library ..............
##      (native routines are registered in init.c)
##      (entry points are symbols with prefix "SC_")

useDynLib(spatstat.model, .registration=TRUE, .fixes="SM_")

## //////////  DO NOT EDIT THE FOLLOWING  ///////////////
## ////////  it is generated automatically  /////////////





# .................................................. 
# Automatically-generated list of documented objects 
# .................................................. 
export("accumulateStatus")
export("active.interactions")
export("adaptcoef")
export("addapply")
export("addROC")
export("addvar")
export("affine.msr")
export("AIC.dppm")
export("AIC.kppm")
export("AIC.mppm")
export("AIC.ppm")
export("anova.mppm")
export("anova.ppm")
export("anova.slrm")
export("areadelta2")
export("AreaInter")
export("as.function.leverage.ppm")
export("as.fv.dppm")
export("as.fv.kppm")
export("as.fv.minconfit")
export("as.im.leverage.ppm")
export("as.interact")
export("as.interact.fii")
export("as.interact.interact")
export("as.interact.ppm")
export("as.interact.zgibbsmodel")
export("as.isf")
export("as.isf.zgibbsmodel")
export("as.layered.msr")
export("as.owin.dppm")
export("as.owin.influence.ppm")
export("as.owin.kppm")
export("as.owin.leverage.ppm")
export("as.owin.msr")
export("as.owin.ppm")
export("as.owin.slrm")
export("as.ppm")
export("as.ppm.dppm")
export("as.ppm.kppm")
export("as.ppm.ppm")
export("as.ppm.profilepl")
export("as.ppm.rppm")
export("as.ppp.influence.ppm")
export("as.tess.rppm")
export("auc.kppm")
export("auc.ppm")
export("auc.slrm")
export("augment.msr")
export("BadGey")
export("bc")
export("bc.ppm")
export("berman.test.ppm")
export("bigvaluerule")
export("blankcoefnames")
export("bt.frame")
export("cannot.update")
export("cauchy.estK")
export("cauchy.estpcf")
export("cdf.test.mppm")
export("cdf.test.ppm")
export("cdf.test.slrm")
export("check.separable")
export("closepaircounts")
export("clusterfield.kppm")
export("clusterfit")
export("clusterkernel.kppm")
export("clusterkernel.zclustermodel")
export("clusterradius.kppm")
export("clusterradius.zclustermodel")
export("clusterstrength")
export("coef.dppm")
export("coef.fii")
export("coef<-.fii")
export("coef.kppm")
export("coef.mppm")
export("coef.ppm")
export("coef.slrm")
export("coef.summary.fii")
export("coef.summary.kppm")
export("coef.summary.ppm")
export("coef.summary.slrm")
export("coef.vblogit") 
export("compareFit")
export("Concom")
export("contour.leverage.ppm")
export("contour.objsurf")
export("crosspaircounts")
export("damaged.ppm")
export("data.mppm")
export("data.ppm")
export("deltasuffstat")
export("detpointprocfamilyfun")
export("deviance.ppm")
export("deviance.slrm")
export("dfbetas.ppm")
export("dfbetas.ppmInfluence")
export("dfbetas.slrm")
export("dffit")
export("dffit.ppm")
export("dffit.slrm")
export("diagnose")
export("diagnose.ppm")
export("diagppmEngine")
export("DiggleGatesStibbard")
export("DiggleGratton")
export("dim.detpointprocfamily")
export("dim.msr")
export("dimnames.msr")
export("domain.dppm")
export("domain.influence.ppm")
export("domain.kppm")
export("domain.leverage.ppm")
export("domain.msr")
export("domain.ppm")
export("domain.slrm")
export("doMultiStraussHard")
export("dppapproxkernel")
export("dppapproxpcf")
export("dppBessel")
export("dppCauchy")
export("dppDpcf")
export("dppeigen")
export("dppGauss")
export("dppkernel")
export("dppm")
export("dppMatern")
export("dppmFixAlgorithm")
export("dppmFixIntensity")
export("dppparbounds")
export("dppPowerExp")
export("DPPSaddle")
export("DPPSaddlePairwise")
export("dppspecden")
export("dppspecdenrange")
export("dropply")
export("dropROC")
export("dummify")
export("dummy.ppm")
export("eem")
export("eem.exactppm")
export("eem.ppm")
export("eem.slrm")
export("effectfun")
export("emend")
export("emend.ppm")
export("emend.slrm")
export("enet.engine")
export("envelope.dppm")
export("envelope.kppm")
export("envelope.ppm")
export("envelope.slrm")
export("equalpairs")          
export("evalInteraction")
export("evalInterEngine")
export("evalPairPotential")
export("evalPairwiseTerm")
export("exactMPLEstrauss")
export("expandDot")
export("extractAIC.dppm")
export("extractAIC.kppm")
export("extractAIC.mppm")
export("extractAIC.ppm")
export("extractAIC.slrm")
export("fakefii")
export("family.hackglmmPQL") 
export("family.vblogit") 
export("fii")
export("Fiksel")
export("fill.coefs")
export("findCovariate")
export("fitin")
export("fitin.ppm")
export("fitin.profilepl")
export("fitted.dppm")
export("fitted.kppm")
export("fitted.mppm")
export("fitted.ppm")
export("fitted.rppm")
export("fitted.slrm")
export("fixef.mppm")
export("flipxy.msr")
export("forbid.logi")
export("formula.dppm")
export("formula.hackglmmPQL") 
export("formula.kppm")
export("formula.ppm")
export("formula.rppm") 
export("formula.slrm")
export("Gcom")
export("getCall.mppm")
export("getdataname")
export("getglmdata")
export("getglmdata.mppm")
export("getglmdata.ppm")
export("getglmdata.slrm")
export("getglmfit")
export("getglmfit.mppm")
export("getglmfit.ppm")
export("getglmfit.slrm")
export("getglmsubset")
export("getglmsubset.mppm")
export("getglmsubset.ppm")
export("getglmsubset.slrm")
export("getppmdatasubset")
export("getppmOriginalCovariates")
export("Geyer")
export("geyercounts")
export("geyerdelta2")
export("GLMpredict")
export("Gres")
export("hackglmmPQL")
export("Hardcore")
export("hardcoredist")
export("hardcoredist.fii")
export("hardcoredist.ppm")
export("harmonic")
export("harmonise.msr")
export("hasglmfit")
export("hasglmfit.mppm")
export("hasglmfit.ppm")
export("hasglmfit.slrm")
export("has.offset")
export("has.offset.term")
export("hierarchicalordering")
export("HierHard")
export("hiermat")
export("hierpair.family")
export("HierStrauss")
export("HierStraussHard")
export("ho.engine")
export("Hybrid")
export("hybrid.family")
export("ic")
export("ic.kppm")
export("ic.ppm")
export("illegal.iformula")
export("image.objsurf")
export("impliedcoefficients")
export("impliedpresence")
export("improve.kppm")
export("[.influence.ppm")
export("influence.ppm")
export("influence.ppmInfluence")
export("influence.slrm")
export("inforder.family")
export("instantiate.interact")
export("integral.influence.ppm")
export("integral.leverage.ppm")
export("integral.msr")
export("intensity.detpointprocfamily")
export("intensity.dppm")
export("intensity.kppm")
export("intensity.mppm")
export("intensity.ppm")
export("intensity.slrm")
export("intensity.zclustermodel")
export("intensity.zgibbsmodel")
export("interactionfamilyname")
export("interactionorder")
export("interactionorder.fii")
export("interactionorder.interact")
export("interactionorder.isf")
export("interactionorder.ppm")
export("interactionorder.zgibbsmodel")
export("intermaker")
export("ippm")
export("is.dppm")
export("is.expandable.ppm")
export("is.hybrid")
export("is.hybrid.interact")
export("is.hybrid.ppm")
export("is.interact")
export("is.kppm")
export("is.lppm")
export("is.marked.mppm")    
export("is.marked.msr")    
export("is.marked.ppm")
export("is.marked.rppm")    
export("is.marked.slrm")
export("is.mppm")
export("is.multitype.mppm")    
export("is.multitype.msr")    
export("is.multitype.ppm")
export("is.multitype.rppm")    
export("is.multitype.slrm")    
export("is.poissonclusterprocess")
export("is.poissonclusterprocess.clusterprocess")
export("is.poissonclusterprocess.default")
export("is.poissonclusterprocess.kppm")
export("is.poissonclusterprocess.zclustermodel")
export("is.poisson.interact")
export("is.poisson.kppm")
export("is.poisson.mppm")
export("is.poisson.ppm")
export("is.poisson.rppm")
export("is.poisson.slrm")
export("is.poisson.zgibbsmodel")
export("is.ppm")
export("is.rppm")
export("is.slrm")
export("is.stationary.detpointprocfamily")
export("is.stationary.dppm")
export("is.stationary.kppm")
export("is.stationary.ppm")
export("is.stationary.slrm")
export("is.stationary.zgibbsmodel")
export("Kcom")
export("killinteraction")
export("Kmodel")
export("Kmodel.clusterprocess")
export("Kmodel.detpointprocfamily")
export("Kmodel.dppm")
export("Kmodel.kppm")
export("Kmodel.ppm")
export("Kmodel.slrm")
export("Kmodel.zclustermodel")
export("Kpcf.kppm")
export("kppm")
export("kppmCLadap")
export("kppmComLik")
export("kppm.formula")
export("kppmMinCon")
export("kppmPalmLik")
export("kppm.ppp")
export("kppm.quad")
export("Kres")
export("labels.dppm")
export("labels.kppm")
export("labels.ppm")
export("labels.slrm")
export("LambertW")
export("LennardJones")
export("leverage")
export("[.leverage.ppm")
export("leverage.ppm")
export("leverage.ppmInfluence")
export("leverage.slrm")
export("lgcp.estK")
export("lgcp.estpcf")
export("lines.objsurf")
export("lines.traj")
export("Lmodel")
export("Lmodel.clusterprocess")
export("Lmodel.detpointprocfamily")
export("Lmodel.dppm")
export("Lmodel.kppm")
export("Lmodel.ppm")
export("Lmodel.slrm")
export("Lmodel.zclustermodel")
export("logi.engine")
export("logLik.dppm")
export("logLik.kppm")
export("logLik.mppm")
export("logLik.ppm")
export("logLik.slrm")
export("logLik.vblogit")
export("LurkEngine")
export("lurking")               
export("lurking.mppm")
export("lurking.ppm")               
export("lurking.ppp")               
export("make.pspace")
export("make.simulrecipe.detpointprocfamily")
export("make.simulrecipe.dppm")
export("make.simulrecipe.kppm")
export("make.simulrecipe.ppm")
export("make.simulrecipe.slrm")
export("mapInterVars")
export("matclust.estK")
export("matclust.estpcf")
export("Mayer")
export("mean.leverage.ppm")
export("measureContinuous")
export("measureDiscrete")
export("measureNegative")
export("measurePositive")
export("measureVariation")
export("measureWeighted")
export("mincontrast")
export("model.covariates")
export("model.depends")
export("model.frame.dppm")
export("modelFrameGam")
export("model.frame.kppm")
export("model.frame.ppm")
export("model.frame.slrm")
export("model.images")
export("model.images.dppm")
export("model.images.kppm")
export("model.images.ppm")
export("model.images.slrm")
export("model.is.additive")
export("model.matrix.dppm")
export("model.matrix.ippm")
export("model.matrix.kppm")
export("model.matrix.mppm")
export("model.matrix.ppm")
export("model.matrix.slrm")
export("model.se.image")
export("mpl.engine")
export("mpl.get.covariates")
export("mpl.prepare")
export("mpl.usable")
export("mppm")
export("[.msr")
export("msr")
export("MultiHard")
export("MultiStrauss")
export("MultiStraussHard")
export("newformula")
export("newstyle.coeff.handling")
export("nndcumfun")
export("nobs.dppm")
export("nobs.kppm")
export("nobs.mppm")
export("nobs.ppm")
export("no.trend.ppm")
export("npfun")
export("objsurf")
export("objsurf.dppm")
export("objsurfEngine")
export("objsurf.kppm")
export("objsurf.minconfit")
export("Ops.msr")
export("optimConverged")
export("optimNsteps")
export("optimStatus")
export("Ord")
export("ord.family")
export("OrdThresh")
export("outdated.interact")
export("oversize.quad")
export("PairPiece")
export("PairPotentialType")
export("pairsat.family")
export("Pairwise")
export("pairwise.family")
export("palmdiagnose")
export("panel.contour")
export("panel.histogram")
export("panel.image")
export("panysib")
export("parameters")
export("parameters.detpointprocfamily")
export("parameters.dppm")
export("parameters.fii")
export("parameters.interact")
export("parameters.kppm")
export("parameters.ppm")
export("parameters.profilepl")
export("parameters.slrm")
export("parres")
export("parres.kppm")
export("parres.ppm")
export("parseRPPM")
export("partialModelMatrix")
export("partialResidualEngine")
export("pcfmodel")
export("pcfmodel.clusterprocess")
export("pcfmodel.detpointprocfamily")
export("pcfmodel.dppm")
export("pcfmodel.kppm")
export("pcfmodel.ppm")
export("pcfmodel.slrm")
export("pcfmodel.zclustermodel")
export("Penttinen")
export("persist")
export("persp.leverage.ppm")
export("persp.objsurf")
export("plot.addvar")
export("plot.diagppm")
export("plot.dppm")
export("ploterodeimage")
export("ploterodewin")
export("plot.fii")
export("plot.influence.ppm")
export("plot.kppm")
export("plot.leverage.ppm")
export("plot.lurk")
export("plot.minconfit")
export("plot.mppm")
export("plot.msr")
export("plot.objsurf")
export("plot.palmdiag")
export("plot.parres")
export("plot.plotppm")
export("plot.ppm")
export("plot.profilepl")
export("plot.qqppm")
export("plot.rppm")
export("plot.slrm")
export("plot.traj")
export("PoisSaddle")
export("PoisSaddleArea")
export("PoisSaddleGeyer")
export("PoisSaddlePairwise")
export("Poisson")
export("PoissonCompareCalc")
export("poisson.fits.better")
export("polynom")
export("ppm")
export("ppmCovariates")
export("ppm.default")
export("ppmDerivatives")
export("ppm.formula")
export("ppmInfluence")
export("ppmInfluenceEngine")
export("PPMmodelmatrix")
export("ppm.ppp")
export("ppm.quad")
export("predict.dppm")
export("predict.kppm")
export("predict.mppm")
export("predict.ppm")
export("predict.profilepl")
export("predict.rppm")
export("predict.slrm")
export("predict.vblogit")
export("predict.zclustermodel")
export("print.addvar")    
export("print.bt.frame")
export("print.detpointprocfamily")
export("print.detpointprocfamilyfun")
export("print.diagppm")
export("print.dppm")
export("print.fii")
export("print.hierarchicalordering")
export("print.influence.ppm")       
export("print.interact")       
export("print.intermaker")       
export("print.isf")
export("print.kppm")
export("print.leverage.ppm")
export("print.lurk")
export("print.minconfit")
export("print.mppm")
export("print.msr")
export("print.objsurf")
export("print.parres")
export("print.plotppm")
export("print.ppm")
export("print.profilepl")
export("print.qqppm")
export("print.rppm")
export("print.slrm")
export("printStatus")
export("printStatusList")
export("print.summary.dppm")
export("print.summary.fii")
export("print.summary.kppm")
export("print.summary.mppm")
export("print.summary.objsurf")
export("print.summary.ppm")
export("print.summary.slrm")
export("print.traj")
export("print.vblogit")
export("print.zclustermodel")
export("print.zgibbsmodel")
export("profilepl")
export("project.ppm")
export("prune.rppm")
export("pseudoR2")
export("pseudoR2.ppm")
export("pseudoR2.slrm")
export("psib")
export("psib.clusterprocess")
export("psib.kppm")
export("psst")
export("psstA")
export("psstG")
export("qqplot.ppm")
export("quadBlockSizes")
export("quad.mppm")
export("quad.ppm")
export("quadrat.test.mppm")
export("quadrat.test.ppm")
export("quadrat.test.slrm")
export("ranef.mppm")
export("rdpp")
export("reach.detpointprocfamily")
export("reach.dppm")
export("reach.fii")
export("reach.interact")
export("reach.kppm")
export("reach.ppm")
export("reach.slrm")
export("reach.zclustermodel")
export("reduceformula")
export("reincarnate.interact")
export("relrisk.ppm")
export("repul")
export("repul.clusterprocess")
export("repul.detpointprocfamily")
export("repul.dppm")
export("repul.kppm")
export("repul.mppm")
export("repul.ppm")
export("repul.rppm")
export("repul.slrm")
export("rescale.msr")
export("resid1panel")
export("resid1plot")
export("resid4plot")
export("residualMeasure")
export("residuals.dppm")
export("residuals.exactppm")
export("residuals.kppm")
export("residuals.mppm")
export("residuals.ppm")
export("residuals.rppm")
export("residuals.slrm")
export("response")
export("response.dppm")
export("response.exactppm")
export("response.glm")
export("response.kppm")
export("response.lm")
export("response.mppm")
export("response.ppm")
export("response.rppm")
export("response.slrm")
export("rex")
export("rhohat.ppm")
export("rhohat.slrm")
export("rmhmodel.ppm")
export("rmh.ppm")
export("roc.dppm")
export("roc.kppm")
export("roc.ppm")
export("roc.slrm")
export("rotate.msr")
export("rppm")
export("SaddleApprox")
export("safeFiniteValue")
export("safePositiveValue")
export("SatPiece")
export("Saturated")
export("scalardilate.msr")
export("shift.influence.ppm")              
export("shift.leverage.ppm")              
export("shift.msr")              
export("signalStatus")
export("simulate.detpointprocfamily")
export("simulate.dppm")
export("simulate.kppm")
export("simulate.mppm")
export("simulate.ppm")
export("simulate.profilepl")
export("simulate.slrm")
export("simulate.zclustermodel")
export("slrAssemblePixelData")
export("slrm")
export("slrmInfluence")
export("slr.prepare")
export("Smooth.influence.ppm")
export("Smooth.leverage.ppm")
export("Smooth.msr")
export("Softcore")
export("spatialCovariateEvidence.dppm")
export("spatialCovariateEvidence.kppm")
export("spatialCovariateEvidence.ppm")
export("spatialCovariateEvidence.slrm")
export("spatialCovariateUnderModel")
export("spatialCovariateUnderModel.dppm")
export("spatialCovariateUnderModel.kppm")
export("spatialCovariateUnderModel.ppm")
export("spatialCovariateUnderModel.slrm")
export("spatstatDPPModelInfo")
export("sp.foundclass")
export("sp.foundclasses")
export("splitHybridInteraction")
export("split.msr")
export("Strauss")
export("strausscounts")
export("StraussHard")
export("stripGLMM")
export("subfits")
export("subfits.new")
export("subfits.old")
export("suffloc")
export("suffstat")
export("suffstat.generic")
export("suffstat.poisson")
export("summary.dppm")
export("summary.fii")
export("summary.kppm")
export("summary.mppm")
export("summary.msr")
export("summary.objsurf")
export("summary.ppm")
export("summary.profilepl")
export("summary.slrm")
export("summary.vblogit")
export("terms.dppm")
export("terms.kppm")
export("terms.mppm")
export("terms.ppm")
export("terms.rppm")
export("terms.slrm")
export("thomas.estK")
export("thomas.estpcf")
export("totalVariation")
export("traj")
export("triplet.family")
export("Triplets")
export("tweak.coefs")
export("unitname.dppm")
export("unitname<-.dppm")
export("unitname.kppm")
export("unitname<-.kppm")
export("unitname.minconfit")
export("unitname<-.minconfit")
export("unitname.msr")
export("unitname<-.msr")
export("unitname.ppm")
export("unitname<-.ppm")
export("unitname.slrm")
export("unitname<-.slrm")
export("unstack.msr")
export("updateData.dppm")
export("updateData.kppm")
export("updateData.ppm")
export("updateData.slrm")
export("update.detpointprocfamily")
export("update.dppm")
export("update.interact")
export("update.ippm")
export("update.kppm")
export("update.msr")
export("update.ppm")
export("update.rppm")
export("update.slrm")
export("valid")
export("valid.detpointprocfamily")
export("valid.ppm")
export("valid.slrm")
export("varcount")
export("varcountEngine")
export("vargamma.estK")
export("vargamma.estpcf")
export("vcov.kppm")
export("vcov.mppm")
export("vcov.ppm")
export("vcov.slrm")
export("versionstring.interact")
export("versionstring.ppm")
export("Window.dppm")
export("Window.influence.ppm")
export("Window.kppm")
export("Window.leverage.ppm")
export("Window.msr")
export("Window.ppm")
export("Window.slrm")
export("windows.mppm")
export("with.msr")
export("zclustermodel")
export("zgibbsmodel")

# .......  Special cases ........... 
S3method("Ops", "msr")
# .......  End of special cases  ... 

# ......................................... 
# Automatically generated list of S3 methods
# ......................................... 
S3method("affine", "msr")
S3method("AIC", "dppm")
S3method("AIC", "kppm")
 S3method("AIC", "mppm")
S3method("AIC", "ppm")
  S3method("anova", "mppm")
  S3method("anova", "ppm")
  S3method("anova", "slrm")
 S3method("as.function", "leverage.ppm")
  S3method("as.fv", "dppm")
  S3method("as.fv", "kppm")
  S3method("as.fv", "minconfit")
  S3method("as.im", "leverage.ppm")
S3method("as.interact", "fii")
S3method("as.interact", "interact")
S3method("as.interact", "ppm")
S3method("as.interact", "zgibbsmodel")
S3method("as.isf", "zgibbsmodel")
 S3method("as.layered", "msr")
 S3method("as.owin", "dppm")
 S3method("as.owin", "influence.ppm")
 S3method("as.owin", "kppm")
 S3method("as.owin", "leverage.ppm")
 S3method("as.owin", "msr")
 S3method("as.owin", "ppm")
 S3method("as.owin", "slrm")
S3method("as.ppm", "dppm")
S3method("as.ppm", "kppm")
S3method("as.ppm", "ppm")
S3method("as.ppm", "profilepl")
S3method("as.ppm", "rppm")
  S3method("as.ppp", "influence.ppm")
S3method("as.tess", "rppm")
S3method("auc", "kppm")
S3method("auc", "ppm")
S3method("auc", "slrm")
  S3method("bc", "ppm")
S3method("berman.test", "ppm")
S3method("cdf.test", "mppm")
S3method("cdf.test", "ppm")
S3method("cdf.test", "slrm")
  S3method("clusterfield", "kppm")
S3method("clusterkernel", "kppm")
S3method("clusterkernel", "zclustermodel")
S3method("clusterradius", "kppm")
 S3method("clusterradius", "zclustermodel")
S3method("coef", "dppm")
S3method("coef", "fii")
S3method("coef", "kppm")
  S3method("coef", "mppm")
  S3method("coef", "ppm")
  S3method("coef", "slrm")
S3method("coef", "summary.fii")
S3method("coef", "summary.kppm")
S3method("coef", "summary.ppm")
S3method("coef", "summary.slrm")
S3method("coef", "vblogit")
 S3method("contour", "leverage.ppm")
S3method("contour", "objsurf")
S3method("deviance", "ppm")
S3method("deviance", "slrm")
S3method("dfbetas", "ppm")
S3method("dfbetas", "ppmInfluence")
S3method("dfbetas", "slrm")
S3method("dffit", "ppm")
S3method("dffit", "slrm")
  S3method("diagnose", "ppm")
  S3method("dim", "detpointprocfamily")
S3method("dim", "msr")
S3method("dimnames", "msr")
 S3method("domain", "dppm")
 S3method("domain", "influence.ppm")
 S3method("domain", "kppm")
 S3method("domain", "leverage.ppm")
 S3method("domain", "msr")
 S3method("domain", "ppm")
 S3method("domain", "slrm")
S3method("eem", "exactppm")
  S3method("eem", "ppm")
  S3method("eem", "slrm")
S3method("emend", "ppm")
S3method("emend", "slrm")
  S3method("envelope", "dppm")
  S3method("envelope", "kppm")
  S3method("envelope", "ppm")
  S3method("envelope", "slrm")
S3method("extractAIC", "dppm")
S3method("extractAIC", "kppm")
 S3method("extractAIC", "mppm")
S3method("extractAIC", "ppm")
S3method("extractAIC", "slrm")
S3method("family", "hackglmmPQL")
S3method("family", "vblogit")
S3method("fitin", "ppm")
S3method("fitin", "profilepl")
  S3method("fitted", "dppm")
  S3method("fitted", "kppm")
 S3method("fitted", "mppm")
  S3method("fitted", "ppm")
S3method("fitted", "rppm")
  S3method("fitted", "slrm")
 S3method("fixef", "mppm")
S3method("flipxy", "msr")
S3method("formula", "dppm")
S3method("formula", "hackglmmPQL")
S3method("formula", "kppm")
S3method("formula", "ppm")
S3method("formula", "rppm")
S3method("formula", "slrm")
 S3method("getCall", "mppm")
S3method("getglmdata", "mppm")
S3method("getglmdata", "ppm")
S3method("getglmdata", "slrm")
S3method("getglmfit", "mppm")
S3method("getglmfit", "ppm")
S3method("getglmfit", "slrm")
S3method("getglmsubset", "mppm")
S3method("getglmsubset", "ppm")
S3method("getglmsubset", "slrm")
S3method("hardcoredist", "fii")
S3method("hardcoredist", "ppm")
S3method("harmonise", "msr")
S3method("hasglmfit", "mppm")
S3method("hasglmfit", "ppm")
S3method("hasglmfit", "slrm")
  S3method("ic", "kppm")
  S3method("ic", "ppm")
S3method("image", "objsurf")
  S3method("[", "influence.ppm")
S3method("influence", "ppm")
S3method("influence", "ppmInfluence")
S3method("influence", "slrm")
 S3method("integral", "influence.ppm")
 S3method("integral", "leverage.ppm")
S3method("integral", "msr")
  S3method("intensity", "detpointprocfamily")
  S3method("intensity", "dppm")
 S3method("intensity", "kppm")
 S3method("intensity", "mppm")
 S3method("intensity", "ppm")
S3method("intensity", "slrm")
 S3method("intensity", "zclustermodel")
S3method("intensity", "zgibbsmodel")
S3method("interactionorder", "fii")
S3method("interactionorder", "interact")
S3method("interactionorder", "isf")
S3method("interactionorder", "ppm")
S3method("interactionorder", "zgibbsmodel")
S3method("is.expandable", "ppm")
S3method("is.hybrid", "interact")
S3method("is.hybrid", "ppm")
S3method("is.marked", "mppm")
S3method("is.marked", "msr")
  S3method("is.marked", "ppm")
S3method("is.marked", "rppm")
S3method("is.marked", "slrm")
S3method("is.multitype", "mppm")
S3method("is.multitype", "msr")
  S3method("is.multitype", "ppm")
S3method("is.multitype", "rppm")
S3method("is.multitype", "slrm")
S3method("is.poissonclusterprocess", "clusterprocess")
S3method("is.poissonclusterprocess", "default")
S3method("is.poissonclusterprocess", "kppm")
S3method("is.poissonclusterprocess", "zclustermodel")
S3method("is.poisson", "interact")
S3method("is.poisson", "kppm")
S3method("is.poisson", "mppm")
S3method("is.poisson", "ppm")
S3method("is.poisson", "rppm")
S3method("is.poisson", "slrm")
S3method("is.poisson", "zgibbsmodel")
S3method("is.stationary", "detpointprocfamily")
S3method("is.stationary", "dppm")
S3method("is.stationary", "kppm")
S3method("is.stationary", "ppm")
S3method("is.stationary", "slrm")
S3method("is.stationary", "zgibbsmodel")
 S3method("Kmodel", "clusterprocess")
   S3method("Kmodel", "detpointprocfamily")
   S3method("Kmodel", "dppm")
   S3method("Kmodel", "kppm")
   S3method("Kmodel", "ppm")
S3method("Kmodel", "slrm")
 S3method("Kmodel", "zclustermodel")
S3method("kppm", "formula")
S3method("kppm", "ppp")
S3method("kppm", "quad")
S3method("labels", "dppm")
S3method("labels", "kppm")
S3method("labels", "ppm")
S3method("labels", "slrm")
  S3method("[", "leverage.ppm")
S3method("leverage", "ppm")
S3method("leverage", "ppmInfluence")
S3method("leverage", "slrm")
S3method("lines", "objsurf")
S3method("lines", "traj")
 S3method("Lmodel", "clusterprocess")
   S3method("Lmodel", "detpointprocfamily")
   S3method("Lmodel", "dppm")
   S3method("Lmodel", "kppm")
   S3method("Lmodel", "ppm")
S3method("Lmodel", "slrm")
 S3method("Lmodel", "zclustermodel")
S3method("logLik", "dppm")
S3method("logLik", "kppm")
 S3method("logLik", "mppm")
S3method("logLik", "ppm")
  S3method("logLik", "slrm")
S3method("logLik", "vblogit")
S3method("lurking", "mppm")
S3method("lurking", "ppm")
S3method("lurking", "ppp")
S3method("make.simulrecipe", "detpointprocfamily")
S3method("make.simulrecipe", "dppm")
S3method("make.simulrecipe", "kppm")
S3method("make.simulrecipe", "ppm")
S3method("make.simulrecipe", "slrm")
 S3method("mean", "leverage.ppm")
 S3method("model.frame", "dppm")
 S3method("model.frame", "kppm")
 S3method("model.frame", "ppm")
 S3method("model.frame", "slrm")
  S3method("model.images", "dppm")
  S3method("model.images", "kppm")
  S3method("model.images", "ppm")
  S3method("model.images", "slrm")
   S3method("model.matrix", "dppm")
   S3method("model.matrix", "ippm")
   S3method("model.matrix", "kppm")
   S3method("model.matrix", "mppm")
   S3method("model.matrix", "ppm")
   S3method("model.matrix", "slrm")
S3method("[", "msr")
S3method("nobs", "dppm")
S3method("nobs", "kppm")
 S3method("nobs", "mppm")
S3method("nobs", "ppm")
S3method("objsurf", "dppm")
S3method("objsurf", "kppm")
S3method("objsurf", "minconfit")
S3method("parameters", "detpointprocfamily")
S3method("parameters", "dppm")
S3method("parameters", "fii")
S3method("parameters", "interact")
S3method("parameters", "kppm")
S3method("parameters", "ppm")
S3method("parameters", "profilepl")
S3method("parameters", "slrm")
S3method("parres", "kppm")
S3method("parres", "ppm")
 S3method("pcfmodel", "clusterprocess")
   S3method("pcfmodel", "detpointprocfamily")
   S3method("pcfmodel", "dppm")
   S3method("pcfmodel", "kppm")
   S3method("pcfmodel", "ppm")
S3method("pcfmodel", "slrm")
 S3method("pcfmodel", "zclustermodel")
 S3method("persp", "leverage.ppm")
S3method("persp", "objsurf")
S3method("plot", "addvar")
  S3method("plot", "diagppm")
  S3method("plot", "dppm")
S3method("plot", "fii")
 S3method("plot", "influence.ppm")
  S3method("plot", "kppm")
 S3method("plot", "leverage.ppm")
S3method("plot", "lurk")
S3method("plot", "minconfit")
  S3method("plot", "mppm")
  S3method("plot", "msr")
S3method("plot", "objsurf")
S3method("plot", "palmdiag")
S3method("plot", "parres")
  S3method("plot", "plotppm")
  S3method("plot", "ppm")
S3method("plot", "profilepl")
S3method("plot", "qqppm")
S3method("plot", "rppm")
  S3method("plot", "slrm")
S3method("plot", "traj")
S3method("ppm", "default")
   S3method("ppm", "formula")
   S3method("ppm", "ppp")
   S3method("ppm", "quad")
  S3method("predict", "dppm")
  S3method("predict", "kppm")
S3method("predict", "mppm")
   S3method("predict", "ppm")
S3method("predict", "profilepl")
S3method("predict", "rppm")
 S3method("predict", "slrm")
S3method("predict", "vblogit")
 S3method("predict", "zclustermodel")
S3method("print", "addvar")
S3method("print", "bt.frame")
S3method("print", "detpointprocfamily")
S3method("print", "detpointprocfamilyfun")
S3method("print", "diagppm")
S3method("print", "dppm")
S3method("print", "fii")
S3method("print", "hierarchicalordering")
S3method("print", "influence.ppm")
S3method("print", "interact")
S3method("print", "intermaker")
S3method("print", "isf")
S3method("print", "kppm")
S3method("print", "leverage.ppm")
S3method("print", "lurk")
S3method("print", "minconfit")
S3method("print", "mppm")
S3method("print", "msr")
S3method("print", "objsurf")
S3method("print", "parres")
S3method("print", "plotppm")
 S3method("print", "ppm")
S3method("print", "profilepl")
S3method("print", "qqppm")
S3method("print", "rppm")
S3method("print", "slrm")
  S3method("print", "summary.dppm")
S3method("print", "summary.fii")
  S3method("print", "summary.kppm")
S3method("print", "summary.mppm")
S3method("print", "summary.objsurf")
  S3method("print", "summary.ppm")
S3method("print", "summary.slrm")
S3method("print", "traj")
S3method("print", "vblogit")
 S3method("print", "zclustermodel")
S3method("print", "zgibbsmodel")
S3method("prune", "rppm")
  S3method("pseudoR2", "ppm")
  S3method("pseudoR2", "slrm")
  S3method("psib", "clusterprocess")
  S3method("psib", "kppm")
  S3method("quadrat.test", "mppm")
S3method("quadrat.test", "ppm")
S3method("quadrat.test", "slrm")
 S3method("ranef", "mppm")
  S3method("reach", "detpointprocfamily")
  S3method("reach", "dppm")
  S3method("reach", "fii")
  S3method("reach", "interact")
  S3method("reach", "kppm")
  S3method("reach", "ppm")
S3method("reach", "slrm")
 S3method("reach", "zclustermodel")
S3method("relrisk", "ppm")
S3method("repul", "clusterprocess")
S3method("repul", "detpointprocfamily")
S3method("repul", "dppm")
S3method("repul", "kppm")
S3method("repul", "mppm")
S3method("repul", "ppm")
S3method("repul", "rppm")
S3method("repul", "slrm")
S3method("rescale", "msr")
  S3method("residuals", "dppm")
S3method("residuals", "exactppm")
  S3method("residuals", "kppm")
S3method("residuals", "mppm")
  S3method("residuals", "ppm")
  S3method("residuals", "rppm")
  S3method("residuals", "slrm")
S3method("response", "dppm")
S3method("response", "exactppm")
S3method("response", "glm")
S3method("response", "kppm")
S3method("response", "lm")
S3method("response", "mppm")
S3method("response", "ppm")
S3method("response", "rppm")
S3method("response", "slrm")
S3method("rhohat", "ppm")
S3method("rhohat", "slrm")
  S3method("rmhmodel", "ppm")
  S3method("rmh", "ppm")
S3method("roc", "dppm")
S3method("roc", "kppm")
S3method("roc", "ppm")
S3method("roc", "slrm")
S3method("rotate", "msr")
S3method("scalardilate", "msr")
S3method("shift", "influence.ppm")
S3method("shift", "leverage.ppm")
S3method("shift", "msr")
  S3method("simulate", "detpointprocfamily")
  S3method("simulate", "dppm")
  S3method("simulate", "kppm")
  S3method("simulate", "mppm")
  S3method("simulate", "ppm")
S3method("simulate", "profilepl")
  S3method("simulate", "slrm")
 S3method("simulate", "zclustermodel")
 S3method("Smooth", "influence.ppm")
 S3method("Smooth", "leverage.ppm")
 S3method("Smooth", "msr")
S3method("spatialCovariateEvidence", "dppm")
S3method("spatialCovariateEvidence", "kppm")
S3method("spatialCovariateEvidence", "ppm")
S3method("spatialCovariateEvidence", "slrm")
S3method("spatialCovariateUnderModel", "dppm")
S3method("spatialCovariateUnderModel", "kppm")
S3method("spatialCovariateUnderModel", "ppm")
S3method("spatialCovariateUnderModel", "slrm")
  S3method("split", "msr")
  S3method("summary", "dppm")
S3method("summary", "fii")
  S3method("summary", "kppm")
S3method("summary", "mppm")
S3method("summary", "msr")
S3method("summary", "objsurf")
  S3method("summary", "ppm")
S3method("summary", "profilepl")
S3method("summary", "slrm")
S3method("summary", "vblogit")
S3method("terms", "dppm")
S3method("terms", "kppm")
 S3method("terms", "mppm")
S3method("terms", "ppm")
S3method("terms", "rppm")
S3method("terms", "slrm")
S3method("unitname", "dppm")
S3method("unitname", "kppm")
S3method("unitname", "minconfit")
S3method("unitname", "msr")
S3method("unitname", "ppm")
S3method("unitname", "slrm")
S3method("unstack", "msr")
S3method("updateData", "dppm")
S3method("updateData", "kppm")
S3method("updateData", "ppm")
S3method("updateData", "slrm")
  S3method("update", "detpointprocfamily")
 S3method("update", "dppm")
S3method("update", "interact")
S3method("update", "ippm")
 S3method("update", "kppm")
S3method("update", "msr")
  S3method("update", "ppm")
 S3method("update", "rppm")
S3method("update", "slrm")
   S3method("valid", "detpointprocfamily")
  S3method("valid", "ppm")
  S3method("valid", "slrm")
   S3method("vcov", "kppm")
   S3method("vcov", "mppm")
  S3method("vcov", "ppm")
  S3method("vcov", "slrm")
 S3method("Window", "dppm")
 S3method("Window", "influence.ppm")
 S3method("Window", "kppm")
 S3method("Window", "leverage.ppm")
 S3method("Window", "msr")
 S3method("Window", "ppm")
 S3method("Window", "slrm")
S3method("with", "msr")
# ......................................... 
#      Assignment methods                   
# ......................................... 
S3method("coef<-", "fii")
S3method("unitname<-", "dppm")
S3method("unitname<-", "kppm")
S3method("unitname<-", "minconfit")
S3method("unitname<-", "msr")
S3method("unitname<-", "ppm")
S3method("unitname<-", "slrm")
# ......................................... 
#      End of methods                       
# ......................................... 
