\name{inforder.family}
\alias{inforder.family}
\title{Infinite Order Interaction Family}
\description{
  An object describing the family of all Gibbs point processes
  with infinite interaction order.
}
\details{
  \bold{Advanced Use Only!}
  
  This structure would not normally be touched by
  the user. It describes the interaction structure
  of Gibbs point processes which have infinite order of interaction,
  such as the area-interaction process \cite{\link{AreaInter}}.
} 
\value{
  Object of class \code{"isf"}, see \code{\link{isf.object}}.
}
\seealso{
  \code{\link{AreaInter}} to create the area interaction process
  structure.
  
  Other families:
  \code{\link{pairwise.family}},
  \code{\link{pairsat.family}},
  \code{\link{ord.family}}.
}
\references{
  Baddeley, A. and Turner, R. (2000)
  Practical maximum pseudolikelihood for spatial point patterns.
  \emph{Australian and New Zealand Journal of Statistics}
  \bold{42}, 283--322.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{models}
